/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_INFO

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>
#include <owl/drivers/owl_board.h>
#include <owl/drivers/owl_uart.h>

#if defined(OWL_CONFIG_MTD)
# include <util/eeprom.h>
#endif

#include <board.h>
#include <compiler.h>
#include <sysclk.h>
#include <pmic.h>
#include <ioport.h>
#include <conf_board.h>

static struct owlboard_data bdata;

int
owlboard_init(void)
{
	pmic_init();
	sysclk_init();
        board_init();
#ifdef CONF_BOARD_ENABLE_USARTF0
	ioport_configure_pin(IOPORT_CREATE_PIN(PORTF, 3), IOPORT_DIR_OUTPUT
                             | IOPORT_INIT_HIGH);
	ioport_configure_pin(IOPORT_CREATE_PIN(PORTF, 2), IOPORT_DIR_INPUT);
#endif
        sysclk_enable_module(SYSCLK_PORT_C, PR_USART0_bm);
        sysclk_enable_module(SYSCLK_PORT_D, PR_USART0_bm);
        sysclk_enable_module(SYSCLK_PORT_F, PR_USART0_bm);
	cpu_irq_enable();

#if BOARD == XMEGA_A1_XPLAINED && EXT_BOARD == SPB800P
        /* Unused pins with UART interface*/
        ioport_configure_pin(PF(0), IOPORT_DIR_INPUT | IOPORT_PULL_UP);
        ioport_configure_pin(PF(4), IOPORT_DIR_INPUT | IOPORT_PULL_UP);
        ioport_configure_pin(PF(7), IOPORT_DIR_INPUT | IOPORT_PULL_UP);
#endif
        
#if defined(OWL_CONFIG_MTD)
        if ((bdata.mtd = eeprom_create()) == NULL) {
                owl_err("fail to init eeprom");
                return OWL_FAILURE;
        }
#endif
        return 0;
}

void
owlboard_mdelay(int ms)
{
        while (ms--)
                _delay_ms(1);
}

void 
owl_putc(char c)
{
/* OWL_BOARD_DEBUG_UART should be initialized by the application */
#if defined(OWL_BOARD_DEBUG_UART)
        if (c == '\n')
                owluart_outb(OWL_BOARD_DEBUG_UART, '\r');

        owluart_outb(OWL_BOARD_DEBUG_UART, c);
#endif
}

struct owlboard_data *
owlboard_data(void)
{
        return &bdata;
}
