#ifndef OWL_DEFS_H
#define OWL_DEFS_H

/* This is the only place where we should include owl_port.h */
#include <owl_port.h>

#ifndef LITTLE_ENDIAN
# define LITTLE_ENDIAN 0x0a0b0c0d /* LSB first */
#endif

#ifndef BIG_ENDIAN
# define BIG_ENDIAN 0x0d0c0b0a /* MSB first */
#endif

#define owl_unused(var) (void) var
#define owl_array_size(a) sizeof(a) / sizeof((a)[0])

#define owl_align(addr, alignment) (((addr) / (alignment)) * (alignment))

#define owl_mbit(n) ((n) * 1024 * 128)
#define owl_mbyte(n) ((n) * 1024 * 1024)
#define owl_kbyte(n) ((n) * 1024)

#define owl_abs(x) ((x) < 0 ? -(x) : (x))
#define owl_min(x, y) ((x) < (y) ? (x) : (y))
#define owl_max(x, y) ((x) > (y) ? (x) : (y))

#define _in_range(c, lo, up)     ((uint8_t) c >= lo && (uint8_t) c <= up)
#define owl_isprint(c)          _in_range(c, 0x20, 0x7f)
#define owl_isdigit(c)          _in_range(c, '0', '9')
#define owl_isxdigit(c)         (isdigit(c) || _in_range(c, 'a', 'f') || \
                                 _in_range(c, 'A', 'F'))
#define owl_islower(c)          _in_range(c, 'a', 'z')
#define owl_isupper(c)          _in_range(c, 'A', 'Z')
#define owl_isspace(c)          (c == ' ' || c == '\f' || c == '\n' || \
                                 c == '\r' || c == '\t' || c == '\v')

/* A port of owl can provide its own owl_bswap16 and owl_bswap32 function. This
   is mainly useful when lots of calls to owl_bswapx are used. Then the code
   size could reduced by having a function rather than a macro for this.
   Also, when building a library that uses owl_defs, we don't want to rely on
   any other external functions than those declared in owl_os.h - so the
   owl_port.h used when building such libraries can have its own implementation
   of bswap */

#if defined(OWL_BSWAP16)
# define owl_bswap16 OWL_BSWAP16
#else
# define owl_bswap16(x) ((((x) << 8) & 0xff00) | \
                        (((x) >> 8) & 0xff))
#endif

#if defined(OWL_BSWAP32)
# define owl_bswap32 OWL_BSWAP32
#else
# define owl_bswap32(x) ((((x) << 24) & 0xff000000) |    \
                         (((x) << 8) & 0xff0000) |       \
                         (((x) >> 8) & 0xff00) |         \
                         (((x) >> 24) & 0xff))
#endif

#define owl_bitset(flags, mask) ((flags) |= (mask))
#define owl_bitclr(flags, mask) ((flags) &= ~(mask))
#define owl_bit(flags, mask)    ((flags) & (mask))

#if BYTE_ORDER == LITTLE_ENDIAN
# define owl_htons(x)           owl_bswap16(x)
# define owl_ntohs(x)           owl_bswap16(x)
# define owl_htonl(x)           owl_bswap32(x)
# define owl_ntohl(x)           owl_bswap32(x)
# define owl_le16_to_cpu(x)     (x)
# define owl_le32_to_cpu(x)     (x)
# define owl_be16_to_cpu(x)     owl_bswap16(x)
# define owl_be32_to_cpu(x)     owl_bswap32(x)
# define owl_cpu_to_le16(x)     (x)
# define owl_cpu_to_le32(x)     (x)
# define owl_cpu_to_be16(x)     owl_bswap16(x)
# define owl_cpu_to_be32(x)     owl_bswap32(x)
#elif BYTE_ORDER == BIG_ENDIAN
# define owl_htons(x)           (x)
# define owl_ntohs(x)           (x)
# define owl_htonl(x)           (x)
# define owl_ntohl(x)           (x)
# define owl_le16_to_cpu(x)     owl_bswap16(x)
# define owl_le32_to_cpu(x)     owl_bswap32(x)
# define owl_be16_to_cpu(x)     (x)
# define owl_be32_to_cpu(x)     (x)
# define owl_cpu_to_le16(x)     owl_bswap16(x)
# define owl_cpu_to_le32(x)     owl_bswap32(x)
# define owl_cpu_to_be16(x)     (x)
# define owl_cpu_to_be32(x)     (x)
#else
# error "BYTE_ORDER not defined"
#endif


#endif
