/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_INFO
#include <owl/core/owl_debug.h>
#include <owl/util/querystring.h>


int
querystring_parse(char *str, struct querystring *q, int num)
{
        int i = 0;

        q[i].key = str;
        
        for (; *str; str++) {

                if (*str == '&') {

                        /* this was the end of the value-token, so put a NULL
                           character here */
                        *str = 0;

                        /* next key-token will follow after '&' */
                        if (i < num)
                                q[i].key = str + 1;
                        
                } else if (*str == '=' && q[i].key) {

                        /* this was the end of the key-token, so put a NULL
                           character here */
                        *str = 0;

                        /* next value-token will follow after '=' */
                        if (i < num) {
                                q[i].value = str + 1;
                                owl_info("%s:%s", q[i].key, q[i].value);
                                i++;
                        }
                }
        }

        return i;
}


char *
querystring_lookup(const char *key, struct querystring *q, int num)
{
        int i;
        char *value = NULL;
        for (i = 0; i < num; i++) {
                if (q[i].key == NULL)
                        break;
                
                if (!owl_os_strcmp(key, q[i].key))
                        value = q[i].value;

                /* continue searching to ensure that we use the last value
                   if we have multiple keys with the same name */
        }

        return value;
}
