/*
 *  OS abstraction layer for wl_api.
 *  Copyright (C) 2010 HD Wireless AB
 *
 *  You should have received a copy of the license along with this library.
 */

#ifndef WL_OS_H
#define WL_OS_H

#include <stdarg.h>
#include <stdlib.h>

#ifdef OWL_CONFIG_MEMTRACE
# define owl_os_alloc(size) \
        owl_memtrace_alloc(size, __FILE__, __LINE__)
# define owl_os_zalloc(size) \
        owl_memtrace_zalloc(size, __FILE__, __LINE__)
# define owl_os_realloc(ptr, size) \
        owl_memtrace_realloc(ptr, size, __FILE__, __LINE__)
# define owl_os_free(p) \
        owl_memtrace_free(p)
#else
# ifdef OWL_CONFIG_MEMCHECK
#  define owl_os_alloc(size) \
         owl_memcheck_alloc(size, __FILE__, __LINE__)
#  define owl_os_zalloc(size) \
         owl_memcheck_zalloc(size, __FILE__, __LINE__)
#  define owl_os_realloc(ptr, size) \
         owl_memcheck_realloc(ptr, size, __FILE__, __LINE__)
#  define owl_os_free(p) \
         owl_memcheck_free(p)
# else
#  define owl_os_alloc owl_mem_alloc
#  define owl_os_zalloc owl_mem_zalloc
#  define owl_os_realloc owl_mem_realloc
#  define owl_os_free owl_mem_free
# endif
#endif

int owl_os_init(void);

void *owl_memtrace_alloc(size_t size, char *file, int line);
void *owl_memtrace_zalloc(size_t size, char *file, int line);
void owl_memtrace_free(void *p);
void *owl_memtrace_realloc(void *p, size_t size, char *file, int line);

void *owl_memcheck_alloc(size_t size, char *file, int line);
void *owl_memcheck_zalloc(size_t size, char *file, int line);
void owl_memcheck_free(void *p);
void *owl_memcheck_realloc(void *p, size_t size, char *file, int line);

void *owl_mem_alloc(size_t size);
void *owl_mem_zalloc(size_t size);
void *owl_mem_realloc(void *ptr, size_t size);
void owl_mem_free(void *p);

void *owl_os_memcpy(void *dst, const void *src, size_t n);
void *owl_os_memset(void *s, int c, size_t n);
void *owl_os_memmove(void *dst, const void *src, size_t n);
size_t owl_os_strlen(const char *s);
char *owl_os_strncpy(char *dst, const char *src, size_t n);
int owl_os_strncmp(const char *s1, const char *s2, size_t n);
int owl_os_strcmp(const char *s1, const char *s2);
char *owl_os_strcpy(char *dst, const char *src);
char *owl_os_strdup(const char *s);
char *owl_os_strndup(const char *s, size_t n);
int owl_os_memcmp(const void *s1, const void *s2, size_t n);
long int owl_os_strtol(const char *nptr, char **endptr, int base);
#define owl_os_atoi(p) owl_os_strtol(p, NULL, 10)
char *owl_os_strchr(const char *s, int c);
char *owl_os_strrchr(const char *s, int c);
int owl_os_strcasecmp(const char *s1, const char *s2);
char *owl_os_strstr(const char *haystack, const char *needle);

int owl_os_snprintf(char *str, size_t size, const char *format, ...);
int owl_os_vsnprintf(char *str, size_t size, const char *format, va_list ap);
    
int owl_os_vprintf(const char *format, va_list arg); /* debug only */
int owl_os_printf(const char *format, ...); /* debug only */


#endif /* WL_OS_H */

