/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_PROTO_H
#define WLP_PROTO_H

#include <stdint.h>

#if !defined(WLP_STACK_OFFLOAD)
# define WLP_STACK_OFFLOAD 1
#endif

/* Message types */
#define WLP_TYPE_MGMT_REQ  0
#define WLP_TYPE_MGMT_CFM  1
#define WLP_TYPE_DATA_REQ  2
#define WLP_TYPE_DATA_CFM  3
#define WLP_TYPE_DATA_IND  4

/* Message id's */
#define WLP_UART_CONFIG 0
#define WLP_POLL        1
#define WLP_FW_VERSION  2
#define WLP_FW_UPGRADE  28
#define WLP_SET_MODE    22
#define WLP_SET_CHANNEL 23
#define WLP_RESET       24
#define WLP_GET_MODE    26
#define WLP_SET_PS_CONF 29
#define WLP_SET_PS_ENABLE 30
#define WLP_SET_CONFIG  31
#define WLP_SET_LED     32
#define WLP_SET_HEARTBEAT 33
#define WLP_SET_POLL_PERIOD 34
#define WLP_GET_CONFIG  35

#define WLP_LINK_UP        3
#define WLP_LINK_DOWN      4
#define WLP_LINK_STATUS    5
#define WLP_LINK_INFO      6

#define WLP_NET_STATUS     7
#define WLP_NET_CONFIG     8
#define WLP_NET_INFO       9
#define WLP_NET_DNSLOOKUP  10
#define WLP_NET_DHCPD      25

#define WLP_SOCK_OPEN    11
#define WLP_SOCK_BIND    12
#define WLP_SOCK_LISTEN  13 
#define WLP_SOCK_CONNECT 14

#define WLP_SOCK_CLOSE  15
#define WLP_SOCK_RECV   16
#define WLP_SOCK_SEND   17
#define WLP_SOCK_SENDTO 18
#define WLP_SOCK_STATUS 19
#define WLP_SOCK_ACCEPT 20
#define WLP_SOCK_INFO   21
#define WLP_SOCK_DATA   27

/*! \ingroup wlp_api */
enum wlp_auth_mode {
        WLP_AUTH_AUTO = 0,
        WLP_AUTH_WEP,
        WLP_AUTH_WPA_PSK,
        WLP_AUTH_WPA2_PSK,
        WLP_AUTH_WPA,
        WLP_AUTH_WPA2,
};

#define WLP_SOCK_STREAM 0
#define WLP_SOCK_DGRAM  1
#define WLP_SOCK_RAW    2

enum wlp_fw_upgrade_method {
        WLP_UPGRADE_METHOD_HTTP_POST
};

#ifndef WL_API_H
#define WL_MAX_PASS_LEN 64
#define WL_SNR_UNKNOWN -128
#define WL_RSSI_UNKNOWN -128
#define WL_SSID_MAX_LENGTH 32
#define WL_MAC_ADDR_LENGTH 6
struct wl_ssid_t {
        char ssid[WL_SSID_MAX_LENGTH];
        uint8_t len;
};

struct wl_mac_addr_t {
        uint8_t octet[WL_MAC_ADDR_LENGTH];
};

#define WL_MODE_STA   0
#define WL_MODE_AP    1
#define WL_MODE_ADHOC 2
#define WL_MODE_WPS   3

enum wl_conn_type_t {
        WL_CONN_TYPE_INFRA, /*!< For infrastructure mode (default) */
        WL_CONN_TYPE_ADHOC  /*!< For ad-hoc mode */
};

#endif /* WL_API_H */

/* Message alignment */
#define WLP_SIZE_ALIGNMENT 4
#define WLP_SIZE_ALIGNMENT_MASK (WLP_SIZE_ALIGNMENT - 1)
#define wlp_pad(len)  \
        (((len) & WLP_SIZE_ALIGNMENT_MASK) ?                            \
         WLP_SIZE_ALIGNMENT - ((len) & WLP_SIZE_ALIGNMENT_MASK) : 0)


#if !defined(__LWIP_IP_ADDR_H__) && !defined(__LWIPOPTS_H__)

/*
 * The struct ip_addr type is based on work from the lwIP network stack.
 *
 */

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification,  are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

struct ip_addr {
        uint32_t addr;
};

extern const struct ip_addr ip_addr_any;

#define IP_ADDR_ANY ((struct ip_addr *) &ip_addr_any)
#define IP4_ADDR(ipaddr, a,b,c,d)                               \
        (ipaddr)->addr = htonl(((uint32_t)((a) & 0xff) << 24) | \
                               ((uint32_t)((b) & 0xff) << 16) | \
                               ((uint32_t)((c) & 0xff) << 8) |  \
                               (uint32_t)((d) & 0xff))
#define ip_addr_set(dest, src) \
        (dest)->addr = ((src) == NULL? 0: (src)->addr)
#define ip_addr_netcmp(addr1, addr2, mask) \
        (((addr1)->addr & (mask)->addr) == ((addr2)->addr & (mask)->addr))
#define ip_addr_cmp(addr1, addr2) \
        ((addr1)->addr == (addr2)->addr)
#define ip_addr_isany(addr1) \
        ((addr1) == NULL || (addr1)->addr == 0)

#define ip4_addr1(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 24) & 0xff)
#define ip4_addr2(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 16) & 0xff)
#define ip4_addr3(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 8) & 0xff)
#define ip4_addr4(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr)) & 0xff)

#endif /* __LWIP_IP_ADDR_H__ */

#define WLP_VERSION(a,b,c) (((uint32_t)((a) & 0xff) << 16) |            \
                            ((uint32_t)((b) & 0xff) << 8) |             \
                            (uint32_t)((c) & 0xff))
#define WLP_MAJOR_VERSION(code) ((uint16_t)((code) >> 16) & 0xff)
#define WLP_MINOR_VERSION(code) ((uint16_t)((code) >> 8) & 0xff)
#define WLP_PATCH_VERSION(code) ((uint16_t)((code)) & 0xff)


/* !\ingroup wlp_api */
#define WLP_CAP_HA_PIN    (1 << 0) /*!< Host Attention pin supported */
#define WLP_CAP_WAKEUP_PIN (1 << 1) /*!< Wakeup pin supported */

struct wlp_uart_config_req {
        uint32_t baudrate;
        uint8_t rtscts;
        uint8_t pad[3];
}; /* sizeof: 8 */

struct wlp_set_config_req {
        uint8_t flags;
#define WLP_FLAG_WAKEUP_PIN_PRESENT 1
        uint8_t pad[3];
}; /* sizeof: 4 */

#if WLP_STACK_OFFLOAD
struct wlp_fw_upgrade_req {
        struct ip_addr addr;
        uint16_t port;
        uint8_t method;
        char pad[1];
#define WLP_PATH_MAX_LENGTH 127
        char path[WLP_PATH_MAX_LENGTH + 1];
}; /* sizeof: 136 */
#endif /* WLP_STACK_OFFLOAD */

struct wlp_set_mode_req {
        uint8_t mode;
        uint8_t pad[3];
}; /* sizeof: 4 */

struct wlp_set_channel_req {
        uint8_t channel;
        uint8_t pad[3];
}; /* sizeof: 4 */

struct wlp_set_ps_conf_req {
        uint32_t ps_delay;
        uint32_t traffic_timeout;
        uint16_t listen_interval;
        uint8_t ps_poll;
        uint8_t rx_all_dtim;
        char pad[3];
}; /* sizeof: 24 */

struct wlp_set_ps_enable_req {
        uint8_t enable;
        uint8_t pad[3];
}; /* sizeof: 4 */

struct wlp_set_led_req {
        uint8_t enable;
        uint8_t pad[3];
}; /* sizeof: 4 */

struct wlp_set_heartbeat_req {
        uint32_t period;
}; /* sizeof: 4 */

struct wlp_set_poll_period_req {
        uint32_t period;
}; /* sizeof: 4 */

struct wlp_link_up_req {
        struct wl_ssid_t ssid;
        char pad[3];
        char key[WL_MAX_PASS_LEN];
        uint8_t auth;
        char pad2[3];
}; /* sizeof: 104 */

#if WLP_STACK_OFFLOAD
struct wlp_net_config_req {
        struct ip_addr ip;
        struct ip_addr netmask;
        struct ip_addr gw;
        struct ip_addr dns_server;
        int8_t dhcp;
        char pad[3];
}; /* sizeof: 24 */

struct wlp_net_dnslookup_req {
#define WLP_HOSTNAME_MAX_LENGTH 127
        char host[WLP_HOSTNAME_MAX_LENGTH + 1];
}; /* sizeof: 128 */

struct wlp_net_dhcpd_req {
        uint8_t enable;
        char pad[3];
}; /* sizeof: 4 */


struct wlp_sock_open_req {
        uint8_t stype;
        uint8_t protocol;
        char pad[2];
}; /* sizeof: 4 */

struct wlp_sock_bind_req {
        struct ip_addr ip;
        int16_t sockid;
        uint16_t port;
}; /* sizeof: 8 */

struct wlp_sock_listen_req {
        int16_t sockid;
        int8_t backlog;
        char pad[1];
}; /* sizeof: 4 */

struct wlp_sock_connect_req {
        struct ip_addr ip;
        int16_t sockid;
        uint16_t port;
}; /* sizeof: 8 */

struct wlp_sock_close_req {
        int16_t sockid;
        char pad[2];
}; /* sizeof: 4 */

struct wlp_sock_recv_req {
        int16_t sockid;
        int16_t datalen;
}; /* sizeof: 4 */

struct wlp_sock_send_req {
        int16_t sockid;
        int16_t datalen;
}; /* sizeof: 4 */

struct wlp_sock_sendto_req {
        struct ip_addr addr;
        int16_t sockid;
        int16_t datalen;
        uint16_t port;
        char pad[2];
}; /* sizeof: 12 */

struct wlp_sock_status_req {
        int16_t sockid;
        char pad[2];
}; /* sizeof: 4 */

struct wlp_sock_accept_req {
        int16_t sockid;
        char pad[2];
}; /* sizeof: 4 */

struct wlp_sock_info_req {
        int16_t sockid;
        char pad[2];
}; /* sizeof: 4 */
#endif /* WLP_STACK_OFFLOAD */

struct wlp_uart_config_cfm {
        int8_t mode; /* operation mode */
        char pad[3];
}; /* sizeof: 4 */

struct wlp_get_config_cfm {
        uint8_t flags;
        uint8_t pad[3];
}; /* sizeof: 4 */

struct wlp_fw_version_cfm {
        uint8_t major; /* WLP_VERSION(a, b, c) */
        uint8_t minor; /* WLP_VERSION(a, b, c) */
        uint8_t patch; /* WLP_VERSION(a, b, c) */
        char pad[1];
        char build[16];
        char hw_rev[16];
}; /* sizeof: 36 */

struct wlp_get_mode_cfm {
        uint8_t mode;
        char pad[3];
}; /* sizeof: 4 */

struct wlp_link_info_cfm {
        struct wl_mac_addr_t hwaddr;
        struct wl_mac_addr_t ap;
        struct wl_ssid_t ssid;
        char pad[3];
        int16_t rssi;
        int16_t snr;
}; /* sizeof: 52 */

#if WLP_STACK_OFFLOAD
struct wlp_net_info_cfm {
        struct ip_addr ip;
        struct ip_addr netmask;
        struct ip_addr gw;
        struct ip_addr dns_server;
        struct ip_addr dns_lookup;

#define WLP_DNS_OK       0
#define WLP_DNS_ERROR   -1
#define WLP_DNS_AGAIN   -2
#define WLP_DNS_INVALID -3
        int8_t dns_result;
        int8_t dhcp;
        char pad[2];
}; /* sizeof: 24 */

struct wlp_sock_status_cfm {
#define WLP_SOCK_STATE_DISCONNECTED 0 /* default */
#define WLP_SOCK_STATE_CONNECTED 1
#define WLP_SOCK_STATE_LISTEN 2
#define WLP_SOCK_STATE_ACCEPT 3
#define WLP_SOCK_STATE_ERROR 4
        int8_t state;
        char pad[3];
}; /* sizeof: 4 */

struct wlp_sock_info_cfm {
        struct ip_addr peer;
}; /* sizeof: 4 */
#endif /* WLP_STACK_OFFLOAD */

struct wlp_hdr {
        uint16_t len;
        uint8_t type;
        uint8_t id;
        int16_t res;
        uint8_t pad;
        char reserved[1];
};

struct wlp_req {
        union {
                struct wlp_uart_config_req uart_config;
                struct wlp_set_config_req config;
#if WLP_STACK_OFFLOAD
                struct wlp_fw_upgrade_req fw_upgrade;
#endif
                struct wlp_set_mode_req set_mode;
                struct wlp_set_channel_req set_channel;
                struct wlp_set_ps_conf_req ps_config;
                struct wlp_set_ps_enable_req ps_enable;
                struct wlp_set_led_req led_enable;
                struct wlp_set_heartbeat_req heartbeat;
                struct wlp_set_poll_period_req poll_period;
                
                struct wlp_link_up_req link_up;
#if WLP_STACK_OFFLOAD
                struct wlp_net_config_req net_config;
                struct wlp_net_dnslookup_req net_dnslookup;
                struct wlp_net_dhcpd_req net_dhcpd;

                struct wlp_sock_open_req sock_open;
                struct wlp_sock_bind_req sock_bind;
                struct wlp_sock_listen_req sock_listen;
                struct wlp_sock_connect_req sock_connect;
                struct wlp_sock_close_req sock_close;
                struct wlp_sock_recv_req sock_recv;
                struct wlp_sock_send_req sock_send;
                struct wlp_sock_sendto_req sock_sendto;
                struct wlp_sock_status_req sock_status;
                struct wlp_sock_accept_req sock_accept;
                struct wlp_sock_info_req sock_info;
#endif
        };
};

struct wlp_cfm {
        union {
                struct wlp_uart_config_cfm uart_config;
                struct wlp_get_config_cfm fw_config;
                struct wlp_fw_version_cfm fw_version;
                struct wlp_get_mode_cfm get_mode;
                
                struct wlp_link_info_cfm link_info;
                
#if WLP_STACK_OFFLOAD
                struct wlp_net_info_cfm net_info;

                struct wlp_sock_status_cfm sock_status;
                struct wlp_sock_info_cfm sock_info;
#endif
        };
};

struct wlp_msg {
        struct wlp_hdr hdr;
        union {
                struct wlp_req req;
                struct wlp_cfm cfm;
                char payload[1];
        };
};

/* API version */
#define WLP_VERSION_CODE WLP_VERSION(1,3,1)

#endif /* WLP_PROTO_H */
